/*****************************************************************/
/*  Executable permettant de recuperer des cliches de la webcam  */
/*****************************************************************/

/** Fichiers d'inclusion **/

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <getopt.h>
#include <signal.h>
#include <sys/stat.h>

#include "../Camera/libcam.h"
#include "../Socket/libsck.h"

/** Constantes **/

#define CHEMIN_SOCKET	"/tmp/webcatch"
#define	MAX_CONNEXIONS	10
#define	MAX_ATTENTE	1000

#define IMAGE_LARGEUR	640
#define IMAGE_HAUTEUR	480

#define	PNG_OUTIL	"pnmtopng"

#define MODE_SERVEUR	0
#define MODE_CLIENT	1

#define MAX_CHAINE	1024

/** Variables globales **/

webcam_technique camera;

/** Initialise la webcam **/

webcam_technique initcam(char *periph){
webcam_technique cam;
int status;
strcpy(cam.periph,periph);
cam.hauteur=IMAGE_LARGEUR;
cam.largeur=IMAGE_HAUTEUR;
status=camOuvrir(&cam);
if(status<0){
  fprintf(stderr,"Impossible d'ouvrir la camera !\n");
  exit(-1);
  }
status=camLuminosite(&cam,90);
if(status<0){
  fprintf(stderr,"Impossible de regler la luminosite !\n");
  }
status=camDemarrer(&cam);
if(status<0){
  fprintf(stderr,"Impossible de demarrer la camera !\n");
  exit(-1);
  }
return cam;
}

/** Generer l'image en format PNG **/

void creerPNG(unsigned char *data,int x,int y)
{
FILE *f=popen(PNG_OUTIL,"w");
fprintf(f,"P6\n");
fprintf(f,"%d %d\n",x,y);
fprintf(f,"255\n");
fwrite(data,x*y*3,1,f);
pclose(f);	
}

/** Arret general **/

void terminer(int signal){
int status;
status=camArreter(&camera);
if(status<0){
  fprintf(stderr,"Impossible d'arreter la camera !\n");
  exit(-1);
  }
status=camFermer(&camera);
if(status<0){
  fprintf(stderr,"Impossible de fermer la camera !\n");
  exit(-1);
  }
}

/** Lire un bloc binaire **/

int lireBloc(int descriptor,unsigned char *bloc,int taille){
int nboctets;
while(taille>0){
  nboctets=read(descriptor,(void *)bloc,taille);
  if(nboctets<=0) return -1;
  bloc += nboctets;
  taille -= nboctets;
  }
return 0;
}

/** Afficher la syntaxe **/

void usage(char *nom){
fprintf(stderr,"Usage:\n");
fprintf(stderr,"  %s -s <periph>\n",nom);
fprintf(stderr,"  %s -c\n",nom);
exit(-1);
}

/** Fonction principale **/

int main(int argc,char *argv[]){

// Lecture des parametres
char *periph="/dev/video0";
int mode=-1;
int flag;
static struct option long_options[]={
  {"server",0,0,'s'},
  {"client",0,0,'c'},
  {0,0,0,0}
  };
while(1){
  flag=getopt_long(argc,argv,"sc",long_options,NULL);
  if(flag==-1) break;
  switch(flag){
    case 's': mode=MODE_SERVEUR; break;
    case 'c': mode=MODE_CLIENT; break;
    default: usage(argv[0]); break;
    }
  }
if(mode<0) usage(argv[0]);
if(mode==MODE_SERVEUR){
  if(argc>optind) periph=argv[optind];
  if(argc>optind+1) usage(argv[0]);
  }
if(mode==MODE_CLIENT){
  if(argc>optind) usage(argv[0]);
  }
 
// Mode serveur
if(mode==MODE_SERVEUR){
  signal(SIGINT,terminer);
  unlink(CHEMIN_SOCKET);
  int ds=initialisationServeur(CHEMIN_SOCKET,MAX_CONNEXIONS);
  chmod(CHEMIN_SOCKET,0777);
  camera=initcam(periph);
  while(1){
    int status;
    status=camCapturer(&camera);
    if(status<0){
      fprintf(stderr,"Impossible de capturer une image !\n");
      return -1;
      }
    status=camImageRGBConversion(&camera);
    if(status<0){
      fprintf(stderr,"Impossible de convertir l'image !\n");
      return -1;
      }
    int dialogue=attenteClient(ds,MAX_ATTENTE);
    if(dialogue<0) continue;
    int nboctets=write(dialogue,&camera.largeur,sizeof(int));
    if(nboctets==sizeof(int)){
      nboctets=write(dialogue,&camera.hauteur,sizeof(int));
      if(nboctets==sizeof(int)){
        int taille_rgb=3*camera.largeur*camera.hauteur;
        nboctets=write(dialogue,camera.rgb,taille_rgb);
        if(nboctets==taille_rgb) close(dialogue);
        }
      }
    }
  }

// Mode client
if(mode==MODE_CLIENT){
  int largeur,hauteur;
  unsigned char *image=NULL;
  int ds=connexionServeur(CHEMIN_SOCKET);
  if(lireBloc(ds,(void *)&largeur,sizeof(int))>=0){
    if(lireBloc(ds,(void *)&hauteur,sizeof(int))>=0 && largeur>0 && hauteur>0){
      int taille_img=3*largeur*hauteur*sizeof(char);
      image=(unsigned char *)malloc(taille_img);
      if(lireBloc(ds,image,taille_img)>=0){
        creerPNG(image,largeur,hauteur);
        close(ds);
        }
      free(image);
      }
    }
  }

return 0;
}
