/** fichier libblue.c **/

/********************************************************/
/** Ce fichier centralise toutes les fonction pour     **/
/** gerer les connexions par sockets Unix              **/
/********************************************************/

/** Fichiers d'inclusion **/

#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/un.h>

#include "libsck.h"

/** Constantes privees **/

/** Fonctions **/

int initialisationServeur(char *chemin,int connexions){
int ds;                         /* Descripteur de SOCKETS */
struct sockaddr_un adresse;     /* Adresse de la SOCKET d'ecoute */
int statut;                     /* Stocke le statut des commandes */

/* Creation de la SOCKET du serveur */
ds=socket(AF_UNIX,SOCK_STREAM,0);
if(ds<0) {perror("initialisationServeur.socket"); exit(-1);}

/* Preparation de la structure adresse du serveur */
adresse.sun_family=AF_UNIX;
strcpy(adresse.sun_path,chemin);

/* Installation du serveur a la bonne adresse */
statut=bind(ds,(struct sockaddr *)&adresse,sizeof adresse);
if(statut<0) {perror("initialisationServeur.bind"); exit(-1);}

/* Initialisation de la file d'attente du serveur    */
statut=listen(ds,connexions);
if(statut<0) {perror("initialisationServeur.listen"); exit(-1);}

return ds;
}

int connexionServeur(char *chemin){
int ds;                         /* Descripteur de SOCKET */
struct sockaddr_un adresse;     /* Adresse de la SOCKET du serveur */
int statut;                     /* Stocke le statut des commandes */

/* Creation de la SOCKET du client */
ds=socket(AF_UNIX,SOCK_STREAM,0);
if(ds<0){perror("connexionServeur.socket"); exit(-1);}

/* Preparation de la structure adresse du serveur */
adresse.sun_family=AF_UNIX;
strcpy(adresse.sun_path,chemin);

/* Connexion au serveur */
statut=connect(ds,(struct sockaddr *)&adresse,sizeof adresse);
if(statut<0){perror("connexionServeur.connect"); exit(-1);}

return ds;
}

int attenteClient(int ds,int timeout){
fd_set rfds;
struct timeval tv;
int retval;

/* Ecoute sur la socket du serveur */
FD_ZERO(&rfds); FD_SET(ds,&rfds);
tv.tv_sec=timeout/1000;
tv.tv_usec=(timeout%1000)*1000;
retval=select(ds+1,&rfds,NULL,NULL,&tv);
if(retval==-1){ perror("attenteClient.select"); exit(-1); }

/* Recuperation de la connexion */
if(FD_ISSET(ds,&rfds)){
  struct sockaddr_un adresse;           /* Adresse du client */
  socklen_t taille=sizeof adresse;      /* Taille de cette adresse */
  int dialogue=accept(ds,(struct sockaddr *)&adresse,&taille);
  return dialogue;
  }

return -1;
}
